==> scrubyt-0.2.3.gem: lib/scrubyt/core/shared/evaluation_context.rb <==
module Scrubyt
  ##
  #=<tt>Holding the evaluation context of the extraction process</tt>
  #
  #Every kind of data that is shared among patterns during the extraction process
  #is held in this class, so it can be looked up anytime.
  #
  #This class provides also some high-level basic functionality in navigation, like 
  #crawling to new pages, attaching doucment to the root pattern once arrived at the
  #desired page etc.
  #
  #It can be viewed as a glue between Extractor and NavigationActions as well - these
  #two classes need to communicate frequently as well as share different information
  #and this is accomplished through EvaluationContext.
  class EvaluationContext
    attr_accessor :root_pattern, :next_page, :document_index, :block_count,
                  :extractor, :limit
    
    def initialize
      @root_pattern = nil
      @next_page = nil
      @block_count = 0
      @document_index = 0
      @extractor = nil
    end
    
    ##
    #Crawl to a new page. This function should not be called from the outside - it is automatically called
    #if the next_page is defined
    def crawl_to_new_page
      temp_document = generate_next_page_link(@next_page)
      return nil if temp_document == nil
      clear_sources_and_sinks(@root_pattern)
      @extractor.fetch(temp_document)
      attach_current_document
    end

    ##
    #Attach document to the root pattern; This is happening automatically as the root pattern is defined or
    #crawling to a new page
    def attach_current_document
      doc = @extractor.get_hpricot_doc
      @root_pattern.filters[0].source << doc
      @root_pattern.filters[0].sink << doc
      @root_pattern.last_result ||= []
      @root_pattern.last_result << doc          
      @root_pattern.result.add_result(@root_pattern.filters[0].source, 
                                      @root_pattern.filters[0].sink)
    end

    ##
    #Based on the given examples, calculate the XPaths for the tree patterns    
    def setup_examples      
      get_root_pattern(nil)
      mark_leaf_parents(@root_pattern)
      generate_examples(@root_pattern)
    end
    
    ##
    #After crawling to the new page, the sources and sinks need to be cleaned
    #since they are no more valid 
    def clear_sources_and_sinks(pattern)
      pattern.filters.each do |filter|
        filter.source = []
        filter.sink = []
      end
      pattern.children.each {|child| clear_sources_and_sinks child}
    end
    
    def generate_next_page_link(example)
      node = XPathUtils.find_node_from_text(@root_pattern.filters[0].source[0], example, true)
      return nil if node == nil
      node.attributes['href'].gsub('&amp;') {'&'}
    end         
        
    def mark_leaf_parents(pattern)
      pattern.children.each { |child|
        pattern.parent_of_leaf = true if child.children.size == 0
      }
      pattern.children.each { |child| mark_leaf_parents(child) }      
    end
    
    def generate_examples(pattern)
      pattern.children.each {|child_pattern| generate_examples(child_pattern) }
      pattern.filters.each { |filter| filter.generate_XPath_for_example } if pattern.type == Pattern::PATTERN_TYPE_TREE
    end    
    
    def get_root_pattern(pattern)
      if @root_pattern == nil
        while (pattern.parent != nil)
          get_root_pattern(pattern.parent)
        end
        @root_pattern = pattern
      end
    end #end of function
  end #end of class EvaluationContext
end #end of module Scrubyt

==> scrubyt-0.2.6.gem: lib/scrubyt/core/shared/evaluation_context.rb <==
module Scrubyt
  ##
  #=<tt>Holding the evaluation context of the extraction process</tt>
  #
  #Every kind of data that is shared among patterns during the extraction process
  #is held in this class, so it can be looked up anytime.
  #
  #This class provides also some high-level basic functionality in navigation, like 
  #crawling to new pages, attaching doucment to the root pattern once arrived at the
  #desired page etc.
  #
  #It can be viewed as a glue between Extractor and NavigationActions as well - these
  #two classes need to communicate frequently as well as share different information
  #and this is accomplished through EvaluationContext.
  class EvaluationContext
    attr_accessor :root_pattern, :document_index, :block_count,
                  :extractor, :uri_builder
    
    def initialize
      @root_pattern = nil
      @next_page = nil
      @block_count = 0
      @document_index = 0
      @extractor = nil
    end
    
    ##
    #Crawl to a new page. This function should not be called from the outside - it is automatically called
    #if the next_page pattern is defined
    def crawl_to_new_page(root_pattern, uri_builder)
      temp_document = uri_builder.next_page_example ? 
                        generate_next_page_link(uri_builder) : 
                        uri_builder.generate_next_uri
      return nil if temp_document == nil
      clear_sources_and_sinks(@root_pattern)
      @extractor.fetch(temp_document)
      attach_current_document
    end

    ##
    #Attach document to the root pattern; This is happening automatically as the root pattern is defined or
    #crawling to a new page
    def attach_current_document
      doc = @extractor.get_hpricot_doc
      @root_pattern.filters[0].source << doc
      @root_pattern.filters[0].sink << doc      
      @root_pattern.last_result ||= []
      @root_pattern.last_result << doc  
      @root_pattern.result.add_result(@root_pattern.filters[0].source, 
                                      @root_pattern.filters[0].sink)
    end

    ##
    #Based on the given examples, calculate the XPaths for the tree patterns    
    def setup_examples      
      get_root_pattern(nil)
      mark_leaf_parents(@root_pattern)
      generate_examples(@root_pattern)
      check_for_multipe_examples(@root_pattern)
    end
    
    ##
    #After crawling to the new page, the sources and sinks need to be cleaned
    #since they are no more valid 
    def clear_sources_and_sinks(pattern)
      pattern.filters.each do |filter|
        filter.source = []
        filter.sink = []
      end
      pattern.children.each {|child| clear_sources_and_sinks child}
    end
    
    def generate_next_page_link(uri_builder)
      uri_builder.next_page_pattern.filters[0].generate_XPath_for_example(true)
      xpath = uri_builder.next_page_pattern.filters[0].xpath
      node = (@extractor.get_hpricot_doc/xpath).map.last
      node = XPathUtils.find_nearest_node_with_attribute(node, 'href')
      return nil if node == nil || node.attributes['href'] == nil      
      node.attributes['href'].gsub('&amp;') {'&'}
    end         
            
    def setup_uri_builder(pattern,args)
      if args[0] =~ /^http.+/
        args.insert(0, @extractor.get_current_doc_url) if args[1] !~ /^http.+/
      end
      @uri_builder = URIBuilder.new(pattern,args)
    end
    
    def get_root_pattern(pattern)
      if @root_pattern == nil
        while (pattern.parent != nil)
          get_root_pattern(pattern.parent)
        end
        @root_pattern = pattern
      end
    end 

private
    def mark_leaf_parents(pattern)
      pattern.children.each { |child|
        pattern.parent_of_leaf = true if child.children.size == 0
      }
      pattern.children.each { |child| mark_leaf_parents(child) }      
    end
    
    ##
    #Check the tree and turn all the XPaths for the examples (but the topmost one)
    #into relative ones
    def check_for_multipe_examples(pattern)
      pattern.children.each {|child_pattern| check_for_multipe_examples(child_pattern) }
      pattern.filters.each { |filter| filter.setup_relative_XPaths } if pattern.type == Pattern::PATTERN_TYPE_TREE    
    end
    
    def generate_examples(pattern)
      pattern.children.each {|child_pattern| generate_examples(child_pattern) }
      pattern.filters.each { |filter| filter.generate_XPath_for_example(false) } if pattern.type == Pattern::PATTERN_TYPE_TREE
    end #end of function generate_examples
  end #end of class EvaluationContext
end #end of module Scrubyt
